function Repository(name, storageStrategy, templateManager, dataSourceHolder, eventEngine, retryCount, timeout, QueryBuilder) {
	var self = this;

	this.retryCount = retryCount;
	this.timeout = timeout;
	var dataSourcesPromise = templateManager.getDataSources();
	var dataSources;
	//just to create event in object
	eventEngine.bindEvents(this, [{
		name: "RepositoryOnRetry",
		code: function() {}
	}]);
	var getFilters = function(query) {
		var filters;
		if (query instanceof Query) {
			filters = query.where();
		} else if (typeof(query) === 'object') {
			filters = QueryBuilder.build().buildQuery(query, "=");
		} else if (query) {
			throw TypeError("Invalid type for query " + typeof(query) + " " + query + ". Use Query or plain object instead.");
		}
		return filters;
	};

	var getWithDataSource = function(name, query, retryCount, timeout, retryCallback) {
		var filters = getFilters(query);
		var promiseDataSource = dataSourceHolder.getDataSource(name);
		return promiseDataSource.then(function(dataSources) {
			dataSources = dataSources || {};
			var itemsPerPage = dataSources.itemsPerPage || 10;
			var page;
			if (itemsPerPage) {
				page = Math.ceil(query.limit() / itemsPerPage);
				itemsPerPage = query.limit() - itemsPerPage;
			}
			return storageStrategy.getDataSource(name, filters, page, itemsPerPage, retryCallback, retryCount || self.retryCount, timeout || self.timeout);
		});
	};

	this.findAll = function(timeout, retryCount, retryCallback) {
		return storageStrategy.getDataSource(name, [], 1, 999999999, retryCallback, retryCount || this.retryCount, timeout || this.timeout);
	};

	this.download = function(query) {
		return storageStrategy.download(name, getFilters(query), undefined, this.retryCount, this.timeout, (function() {
			this.onRetry();
		}).bind(this));
	};

	this.find = function(query, retryCount, timeout, retryCallback) {
		var promise;
		if (!dataSources) {
			promise = dataSourcesPromise.then(function(data) {
				dataSources = data;
				return getWithDataSource(name, query, retryCount, timeout, retryCallback);
			});
		} else {
			promise = getWithDataSource(name, query, retryCount, timeout, retryCallback);
		}
		return promise;
	};

	this.findOne = function(query, retryCount, timeout, retryCallback) {
		query = query ? query : new Query();
		query.limit(1);
		query.offset(0);
		var promise;
		if (!dataSources) {
			promise = dataSourcesPromise.then(function(data) {
				dataSources = data;
				return getWithDataSource(name, query, retryCount, timeout, retryCallback);
			});
		} else {
			promise = getWithDataSource(name, query, retryCount, timeout, retryCallback);
		}
		return promise.then(function(all) {
			return Object.prototype.toString.call(all) === '[object Array]' ? all[0] : all;
		});
	};

	this.save = function(data) {
		return storageStrategy.save(name, data);
	};

	this.saveInServer = function(dataSource) {
		return storageStrategy.saveInServer(name, null, dataSource, this.retryCount, this.timeout, (function() {
			this.onRetry();
		}).bind(this));
	};

	this.removeFromServer = function(row) {
		return storageStrategy.removeFromServer(name, null, row, this.retryCount, this.timeout, (function() {
			this.onRetry();
		}).bind(this));
	};

	this.remove = function(query) {
		return storageStrategy.remove(name, getFilters(query));
	};

	this.clearAll = function() {
		return storageStrategy.remove(name, []);
	};

}